/*
 * 03.08.2018
 * CoCoNut
 * Developed by: Mattia Siragusa - masir@dtu.dk
 * (Fiji Is Just) ImageJ 2.0.0-rc-59/1.51n
 * Java 1.8.0_66
 * Running on Windows 10 64bit
 */

//-----------------------------------------------PRELIMINARY STEPS---------------------------------------------------------//
/*SET INPUTS*/
	path="C:";

	Dialog.create("CoCoNut   v.1.0.0   Support: masir@dtu.dk");
	Dialog.addMessage("Warning: Rename the samples as 1.jpg, 2.jpg, etc. Rename the control as background.jpg"); 
	Dialog.addString("Where are the images? Full path:", path);
	Dialog.addNumber("Number of images to be processed:", 15);
	Dialog.addNumber("Radius of the smallest colony (in pixel):", 19);
	Dialog.show();
	
	path= Dialog.getString();

	//Correct path by adding "\\" at the end if missing)
	l=lengthOf(path);
	lastletter=substring (path,l-1); 
	if (lastletter != "/") {
	path=path+"\\";
	}
	
	nrbilder=Dialog.getNumber();
	smallestColonyp=Dialog.getNumber();

	print("Experiment: "+path);
	print("Radius: "+smallestColonyp+" pixels");
	print(" ");
	
//---------------------------------------------------BACKGROUND------------------------------------------------------------//
/*OPEN background.jpg AND ADD A ROI*/
	open(path+"background.jpg");
	
	run("ROI Manager...");
	waitForUser ("Draw or import a ROI, add it to the ROI manager, then press OK"); 
	roiManager("save", path+"saved_ROI.zip");
	type=selectionType();
	
	setBatchMode(true); //true = don't show intermediate results

/*KEEP GREEN CHANNEL AND CROP ROI*/	
	WindowTitle=getTitle(); //of the background
	KeepGreenChannel();	 	//split colors and discard blule+red channels
	
	SelectFlask();			//crop ROI

/*NORMALIZATION: DIVIDE THE BACKGROUND BY A WHITE IMAGE*/
	//Create white image
	run("Select All");
	getSelectionBounds(x, y, width, height);
	newImage("white255", "8-bit white", width, height, 1);
	
	//Normalization
	imageCalculator("Divide create 32-bit", "background.jpg (green)", "white255");
	
	selectWindow("Result of background.jpg (green)");
	setMinAndMax(0, 1);
	close("background.jpg (green)");

//---------------------------------------------------SAMPLES---------------------------------------------------------------//
/*OPEN THE SAMPLES AND PROCEDE WITH CALCULATIONS*/
	for (p=1; p<nrbilder+1; p++)	{
	open(path+p+".jpg");

	/*KEEP GREEN CHANNEL AND CROP ROI*/
		WindowTitle=getTitle(); //of the sample
		KeepGreenChannel();	 //split colors and discard blule+red channels
		roiManager("select",0); //select original ROI
		SelectFlask();			//crop ROI
		
		if (p==1) {
			Roi.setName("CroppedSelection"); 
			roiManager("Add"); 
		} //if

	/*NORMALIZATION: DIVIDE THE SAMPLES BY A WHITE IMAGE*/	
		//Normalization
		selectWindow(p+".jpg (green)");
		WindowTitle=getTitle();		
		imageCalculator("Divide create 32-bit", WindowTitle, "white255");
		selectWindow("Result of "+p+".jpg (green)");
		setMinAndMax(0, 1);
		close(p+".jpg (green)");
	
	/*DIVISION: BACKGROUND REMOVAL*/
		selectWindow("Result of background.jpg (green)");
		selectWindow("Result of "+p+".jpg (green)");
		WindowTitle=getTitle();
		imageCalculator("Divide create 32-bit", WindowTitle, "Result of background.jpg (green)"); //sample divided by background

	/*SCALE BACK FROM 0-1 TO 0-255*/	
	//hint: Colonies are in the left tail of histogram.					
		selectWindow("Result of Result of "+p+".jpg (green)");
		for(x=0;x<width;x++){ 
			for(y=0;y<height;y++){ 
				pixel=getPixel(x,y); 
				//values greater than 1: they are on the edge of the flask or are due to stains in the control flask.
				if (pixel>0.95) {pixel=1;}				
				setPixel(x,y,255*pixel);					 
			}//for y
		}//for x
		selectWindow("Result of Result of "+p+".jpg (green)");
		setMinAndMax(0, 255); 
				
	/*CLOSE UNUSED WINDOWS*/
		close(WindowTitle);

	/*NOISE REDUCTION: MORPHOLOGICAL FILTERS (OPENING)*/
	//hint: colonies must be valleys of the the inverted umbra.
		run("Maximum...", smallestColonyp); //dilation of the umbra, erosion of the inverted umbra. Radius in pixels
		run("Minimum...", smallestColonyp); //viceversa

	/*OVERLAY WHITE SELECTION*/
		roiManager("select",1); 

		if (type != 0){ 
			run("Make Inverse");
			setForegroundColor(255, 255, 255);
			run("Fill", "slice");
			roiManager("select",0);
		}//if


	/*THRESHOLDING*/
		setAutoThreshold("Default");
		setOption("BlackBackground", false);
		run("Convert to Mask");	
		run("Make Binary");

	/*WATERSHED*/
		run("Watershed");

	/*SAVE AND CLOSE*/
		saveAs("Jpeg", path+"binary"+p+".jpg");
		close("binary"+p+".jpg");
	
	}//initial for

/*CLOSE UNUSED WINDOWS*/	
	close("Result of background.jpg (green)");
	close("white255");	
	print("All samples have been divided by background.jpg. Calculations are performed only for the Green channel.");
	print(" ");

//----------------------------------------------ANALYZE PARTICLES----------------------------------------------------------//

/*******************************************/
/*************** FIRST ROUND ***************/
/*******************************************/

print("****************************************************************************************");
print("First counting: Round colonies with area between minimum and infinity");
print("****************************************************************************************");

/*OPEN BINARY SAMPLES AND ANALYZE PARTICLES (AGAIN)*/
	total_counts = newArray(nrbilder); //array for summarizing results
	avgArea = newArray(nrbilder);
	
	for (p=1; p<nrbilder+1; p++)	{
	
		open(path+"binary"+p+".jpg");
		run("Make Binary");
	
		//the morphological filter removed what shouldn't be counted because of its size, but we used watershed.
		run("Set Measurements...", "area standard perimeter redirect=None decimal=2");
		smallestColonyAreap=smallestColonyp*smallestColonyp*3.14;
		run("Analyze Particles...", "size="+smallestColonyAreap+"-infinity circularity=0.50-1.00 display clear summarize in_situ add");
	
		count = nResults(); //number of counts
	
		//Results into an array:
		resultsArray=newArray(); 
		for(i=0;i<count;i++){ 
			resultsArray=Array.concat(resultsArray,getResult("Area",i)); 
		} //for 
	
		//Obtain average area information
		Array.getStatistics(resultsArray,min,max,avg,stDev);
		avgArea[p-1]=avg;
		//print("The average area of sample "+p+" is: "+avgArea[p-1]);
	   
		//Decide if corrections are necessary
		adjustment=0;
		//go through all the results of a single binary picture:
		for(i=0;i<count;i++){ 
			ratio=round(resultsArray[i]/avgArea[p-1]);

			//Adjust counts according to colony area and rename ROIs		
			if (ratio<1) {
				adjustment=adjustment+1;
				roiManager("Select", i);
				roiManager("Rename", "1");
			} //if
		
			if (ratio>=1) {
				adjustment=adjustment+ratio;
				roiManager("Select", i);
				roiManager("Rename", ratio);
			} //if
			
		} //for

		roiManager("Show All with labels");
		run("Labels...", "color=white font=30 show use");
		run("Flatten"); //Creates image with labels

		//Save and close
		saveAs("Jpeg", path+p+" First counting.jpg");
		close(p+" First counting.jpg");
	
		total_counts[p-1]=total_counts[p-1]+adjustment;	 
		print("Sample "+p+", colonies detected: "+total_counts[p-1]);
		   close("binary"+p+".jpg");
		   		
	}//for

/*CLOSE UNUSED WINDOWS*/	

	selectWindow("Results");
	run("Close");
	selectWindow("Summary"); 
	run("Close");

/*******************************************/
/************** SECOND ROUND ***************/
/*******************************************/

print(" ");
print("****************************************************************************************");
print("Second counting: Big colonies with strange morphology");
print("****************************************************************************************");

/*OPEN BINARY SAMPLES AND ANALYZE PARTICLES (AGAIN)*/

	for (p=1; p<nrbilder+1; p++)	{
	
		open(path+"binary"+p+".jpg");
		run("Make Binary");

		roiManager("reset");
		run("Analyze Particles...", "size="+avgArea[p-1]+"-infinity circularity=0.00-0.49 show=Nothing display clear summarize in_situ add");
		
		count = nResults(); //number of counts
	   
		//---------------------AREA CORRECTION-----------------------//
		resultsArray=newArray(); 
	
		adjustment_uncounted=0;
		   for(i=0;i<count;i++){ 
			particleArea=getResult("Area",i); 
			ratio=round(particleArea/avgArea[p-1]);
			if (ratio<1) {
				adjustment_uncounted=adjustment_uncounted+1;
				roiManager("Select", i);
				roiManager("Rename", "1");
			} //if
	
			if (ratio>=1) {
				adjustment_uncounted=adjustment_uncounted+ratio;
				roiManager("Select", i);
				roiManager("Rename", ratio);
			} //if
		   }//for
			
		 roiManager("Show All with labels");
		 run("Labels...", "color=white font=30 show use");
		 run("Flatten"); //Creates image with labels
	
		 //Save and close
		 saveAs("Jpeg", path+p+" Second counting.jpg");
		 close(p+" Second counting.jpg");
	
		 total_counts[p-1]=total_counts[p-1]+adjustment_uncounted;
		 print("Add to "+"Sample "+p+" a total of: "+adjustment_uncounted+" counts");	
		 close("binary"+p+".jpg");	  
 
	}//for


print(" ");
print("****************************************************************************************");
print("Total counts");
print("****************************************************************************************");

for (p=1; p<nrbilder+1; p++)	{
	print("Sample "+p+", total colonies detected: "+total_counts[p-1]);
}

f="Results";		 
if (isOpen(f)){ 
   selectWindow("Results");
   run("Close"); 
}   
selectWindow("Summary"); 
run("Close");  
selectWindow("ROI Manager");
run("Close");

selectWindow("Log"); 
saveAs("Text", path+"Log");

/*******************************************/
/******* CREATE OVERLAY OUTPUT IMAGE *******/
/*******************************************/

setBatchMode(false); //true = don't show intermediate results	

for (p=1; p<nrbilder+1; p++)	{
	open(path+p+" First counting.jpg");	
	selectWindow(p+" First counting.jpg");
	WindowTitle1=getTitle();	
	open(path+p+" Second counting.jpg");	
	selectWindow(p+" Second counting.jpg");
	WindowTitle2=getTitle();
	
	imageCalculator("Add create", WindowTitle1,WindowTitle2); //Add First to Second counting

	selectWindow(p+" First counting.jpg");
	close();
	selectWindow(p+" Second counting.jpg");
	close();

	open(path+p+".jpg");
	selectWindow(p+".jpg");
	
	roiManager("open", path+"saved_ROI.zip");
	roiManager("Show All without labels");
	roiManager("Show All without labels");
	roiManager("Select", 0);	
	run("Crop");

	run("Add Image...", "image=[Result of "+p+" First counting.jpg] x=0 y=0 opacity=30");
	saveAs("Jpeg", path+p+" Results.jpg");

	selectWindow("Result of "+p+" First counting.jpg");
	run("Close");
	selectWindow("ROI Manager");
	run("Close");
}

if (nrbilder > 1) {	
	run("Images to Stack", "name=Results title=[] use"); //Final results image to 1 stack
}

//--------------------------------------------AVAILABLE FUNCTIONS----------------------------------------------------------//

function SelectFlask()	  {
							 roiManager("Select", 0);
							 run("Clear Outside");
							 run("Crop");

							 //If ROI is a square (type=0) corrections are needed.
							 pixel_check=getPixel(1,1); 
							 if (type == 0){ 
								 setPixel(1,1,0);
 //If squared ROI outside is not set to black (no outside, just cropped)
								 run("Select All");
							 } //if 
							 
							}

function KeepGreenChannel() {
							run("Split Channels");
							close(WindowTitle+" (red)");
							close(WindowTitle+" (blue)");
							}
